==> clean_android_design-0.0.180.tar.gz: lib/src/structure/list/scroll_bar_widget.dart <==
import 'dart:async';
import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class ScrollBarWidget extends StatefulWidget {
  final ScrollController scrollController;

  final DefaultScaffoldScrollBarConfiguration scrollBarConfiguration;
  final bool cutBottom;

  const ScrollBarWidget({
    Key key,
    @required this.scrollController,
    @required this.scrollBarConfiguration,
    this.cutBottom = false,
  }) : super(key: key);

  @override
  _ScrollBarWidgetState createState() => _ScrollBarWidgetState();
}

class _ScrollBarWidgetState extends State<ScrollBarWidget> {
  bool scrollBarVisible = false;
  Timer scrollBarTimer;

  double indicatorPosTop;
  int selectedSectionIndex;

  double maxIndicatorHeight;
  double minIndicatorHeight = 21;

  @override
  void initState() {
    super.initState();

    widget.scrollController.addListener(() {
      if (!scrollBarVisible) setState(() => scrollBarVisible = true);
      scrollBarTimer?.cancel();
      scrollBarTimer = Timer(
          Duration(milliseconds: 2000),
          () =>
              {if (scrollBarVisible) setState(() => scrollBarVisible = false)});
    });
  }

  void _setIndicator(double y) {
    setState(() {
      indicatorPosTop = max(min(y, maxIndicatorHeight), minIndicatorHeight);

      var percent = (indicatorPosTop - minIndicatorHeight) /
          (maxIndicatorHeight - minIndicatorHeight);
      var newSelectedSectionIndex =
          ((widget.scrollBarConfiguration.sectionItems.length - 1) * percent)
              .round();

      if (selectedSectionIndex != newSelectedSectionIndex)
        widget.scrollController.animateTo(
          widget
              .scrollBarConfiguration.sectionPositions[newSelectedSectionIndex],
          duration: Duration(milliseconds: 100),
          curve: Curves.linear,
        );

      selectedSectionIndex = newSelectedSectionIndex;
    });

    scrollBarTimer?.cancel();
    scrollBarTimer = Timer(Duration(milliseconds: 2000),
        () => {if (scrollBarVisible) setState(() => scrollBarVisible = false)});
  }

  @override
  Widget build(BuildContext context) {
    maxIndicatorHeight =
        MediaQuery.of(context).size.height - 59 - (widget.cutBottom ? 60 : 0);

    return Stack(
      children: [
        Align(
            alignment: Alignment.centerRight,
            child: AnimatedPadding(
              duration: Duration(milliseconds: 800),
              padding: EdgeInsets.only(
                  right: 10,
                  top: 20,
                  bottom: 15,
                  left: scrollBarVisible
                      ? 0
                      : (MediaQuery.of(context).size.width + 5)),
              child: GestureDetector(
                onVerticalDragUpdate: (details) =>
                    _setIndicator(details.localPosition.dy),
                onVerticalDragEnd: (details) =>
                    setState(() => indicatorPosTop = null),
                onTapDown: (details) {
                  selectedSectionIndex = null;
                  _setIndicator(details.localPosition.dy);
                },
                onTapUp: (details) => setState(() => indicatorPosTop = null),
                child: Container(
                  decoration: BoxDecoration(
                    color: CleanAndroidDesignDefaultColors.defaultColorLight
                        .withOpacity(0.7),
                    borderRadius: BorderRadius.circular(10),
                  ),
                  width: 22,
                  padding: EdgeInsets.symmetric(vertical: 3),
                  child: DefaultTextStyle(
                    style: TextStyles.defaultTextStyle().copyWith(
                      fontSize: 12,
                      fontWeight: FontWeight.w400,
                    ),
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: widget.scrollBarConfiguration.sectionItems,
                    ),
                  ),
                ),
              ),
            )),
        indicatorPosTop == null || !scrollBarVisible
            ? Container()
            : AnimatedPositioned(
                duration: Duration(milliseconds: 50),
                right: 11,
                top: indicatorPosTop,
                child: Container(
                  width: 20,
                  height: 45,
                  decoration: BoxDecoration(
                    color:
                        CleanAndroidDesignDefaultColors.accent.withOpacity(0.6),
                    borderRadius: BorderRadius.circular(10),
                  ),
                ),
              ),
        indicatorPosTop == null || !scrollBarVisible
            ? Container()
            : AnimatedPositioned(
                duration: Duration(milliseconds: 50),
                right: 50,
                top: indicatorPosTop,
                child: Container(
                  width: widget.scrollBarConfiguration
                          .groupScrollBarIndicatorWidth ??
                      45,
                  height: 45,
                  decoration: BoxDecoration(
                    color:
                        CleanAndroidDesignDefaultColors.accent.withOpacity(0.6),
                    borderRadius: BorderRadius.circular(50),
                  ),
                  child: Center(
                    child: DefaultTextStyle(
                        style: TextStyles.defaultTextStyle().copyWith(
                          color: Colors.white,
                          fontSize: 21,
                        ),
                        child: widget.scrollBarConfiguration
                                    .sectionIndicatorItems ==
                                null
                            ? widget.scrollBarConfiguration
                                .sectionItems[selectedSectionIndex]
                            : widget.scrollBarConfiguration
                                .sectionIndicatorItems[selectedSectionIndex]),
                  ),
                ),
              )
      ],
    );
  }
}

@immutable
class ListScaffoldScrollBarConfiguration<T> {
  final Widget Function(T) groupScrollBarWidget;
  final Widget Function(T) groupScrollBarIndicatorWidget;
  final double groupScrollBarIndicatorWidth;
  final double scrollBarOffset;
  final double sectionHeaderHeight;
  final double listItemHeight;

  ListScaffoldScrollBarConfiguration({
    this.groupScrollBarIndicatorWidth,
    this.groupScrollBarIndicatorWidget,
    this.groupScrollBarWidget,
    this.scrollBarOffset = 0.0,
    this.sectionHeaderHeight = 26.9,
    this.listItemHeight = 59.1,
  });
}

@immutable
class DefaultScaffoldScrollBarConfiguration<T>
    extends ListScaffoldScrollBarConfiguration<T> {
  final List<Widget> sectionItems;
  final List<Widget> sectionIndicatorItems;
  final List<double> sectionPositions;
  final ScrollController scrollController;

  DefaultScaffoldScrollBarConfiguration({
    groupScrollBarIndicatorWidth,
    @required this.sectionItems,
    this.sectionIndicatorItems,
    @required this.sectionPositions,
    this.scrollController,
  }) : super(groupScrollBarIndicatorWidth: groupScrollBarIndicatorWidth);
}

==> clean_android_design-0.0.181.tar.gz: lib/src/structure/list/scroll_bar_widget.dart <==
import 'dart:async';
import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class ScrollBarWidget extends StatefulWidget {
  final ScrollController scrollController;

  final DefaultScaffoldScrollBarConfiguration scrollBarConfiguration;
  final bool cutBottom;

  const ScrollBarWidget({
    Key key,
    @required this.scrollController,
    @required this.scrollBarConfiguration,
    this.cutBottom = false,
  }) : super(key: key);

  @override
  _ScrollBarWidgetState createState() => _ScrollBarWidgetState();
}

class _ScrollBarWidgetState extends State<ScrollBarWidget> {
  bool scrollBarVisible = false;
  Timer scrollBarTimer;

  double indicatorPosTop;
  int selectedSectionIndex;

  double maxIndicatorHeight;
  double minIndicatorHeight = 21;

  @override
  void initState() {
    super.initState();

    widget.scrollController.addListener(() {
      if (!scrollBarVisible) setState(() => scrollBarVisible = true);
      scrollBarTimer?.cancel();
      scrollBarTimer = Timer(
          Duration(milliseconds: 2000),
          () =>
              {if (scrollBarVisible) setState(() => scrollBarVisible = false)});
    });
  }

  void _setIndicator(double y) {
    setState(() {
      indicatorPosTop = max(min(y, maxIndicatorHeight), minIndicatorHeight);

      var percent = (indicatorPosTop - minIndicatorHeight) /
          (maxIndicatorHeight - minIndicatorHeight);
      var newSelectedSectionIndex =
          ((widget.scrollBarConfiguration.sectionItems.length - 1) * percent)
              .round();

      if (selectedSectionIndex != newSelectedSectionIndex)
        widget.scrollController.animateTo(
          widget
              .scrollBarConfiguration.sectionPositions[newSelectedSectionIndex],
          duration: Duration(milliseconds: 100),
          curve: Curves.linear,
        );

      selectedSectionIndex = newSelectedSectionIndex;
    });

    scrollBarTimer?.cancel();
    scrollBarTimer = Timer(Duration(milliseconds: 2000),
        () => {if (scrollBarVisible) setState(() => scrollBarVisible = false)});
  }

  @override
  Widget build(BuildContext context) {
    maxIndicatorHeight = MediaQuery.of(context).size.height -
        (SplitNavigator.singleNavigator(context) ? 146 : 59) -
        (widget.cutBottom ? 60 : 0);

    return Stack(
      children: [
        Align(
            alignment: Alignment.centerRight,
            child: AnimatedPadding(
              duration: Duration(milliseconds: 800),
              padding: EdgeInsets.only(
                  right: 10,
                  top: 20,
                  bottom: 15,
                  left: scrollBarVisible
                      ? 0
                      : (MediaQuery.of(context).size.width + 5)),
              child: GestureDetector(
                onVerticalDragUpdate: (details) =>
                    _setIndicator(details.localPosition.dy),
                onVerticalDragEnd: (details) =>
                    setState(() => indicatorPosTop = null),
                onTapDown: (details) {
                  selectedSectionIndex = null;
                  _setIndicator(details.localPosition.dy);
                },
                onTapUp: (details) => setState(() => indicatorPosTop = null),
                child: Container(
                  decoration: BoxDecoration(
                    color: CleanAndroidDesignDefaultColors.defaultColorLight
                        .withOpacity(0.7),
                    borderRadius: BorderRadius.circular(10),
                  ),
                  width: 22,
                  padding: EdgeInsets.symmetric(vertical: 3),
                  child: DefaultTextStyle(
                    style: TextStyles.defaultTextStyle().copyWith(
                      fontSize: 12,
                      fontWeight: FontWeight.w400,
                    ),
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: widget.scrollBarConfiguration.sectionItems,
                    ),
                  ),
                ),
              ),
            )),
        indicatorPosTop == null || !scrollBarVisible
            ? Container()
            : AnimatedPositioned(
                duration: Duration(milliseconds: 50),
                right: 11,
                top: indicatorPosTop,
                child: Container(
                  width: 20,
                  height: 45,
                  decoration: BoxDecoration(
                    color:
                        CleanAndroidDesignDefaultColors.accent.withOpacity(0.6),
                    borderRadius: BorderRadius.circular(10),
                  ),
                ),
              ),
        indicatorPosTop == null || !scrollBarVisible
            ? Container()
            : AnimatedPositioned(
                duration: Duration(milliseconds: 50),
                right: 50,
                top: indicatorPosTop,
                child: Container(
                  width: widget.scrollBarConfiguration
                          .groupScrollBarIndicatorWidth ??
                      45,
                  height: 45,
                  decoration: BoxDecoration(
                    color:
                        CleanAndroidDesignDefaultColors.accent.withOpacity(0.6),
                    borderRadius: BorderRadius.circular(50),
                  ),
                  child: Center(
                    child: DefaultTextStyle(
                        style: TextStyles.defaultTextStyle().copyWith(
                          color: Colors.white,
                          fontSize: 21,
                        ),
                        child: widget.scrollBarConfiguration
                                    .sectionIndicatorItems ==
                                null
                            ? widget.scrollBarConfiguration
                                .sectionItems[selectedSectionIndex]
                            : widget.scrollBarConfiguration
                                .sectionIndicatorItems[selectedSectionIndex]),
                  ),
                ),
              )
      ],
    );
  }
}

@immutable
class ListScaffoldScrollBarConfiguration<T> {
  final Widget Function(T) groupScrollBarWidget;
  final Widget Function(T) groupScrollBarIndicatorWidget;
  final double groupScrollBarIndicatorWidth;
  final double scrollBarOffset;
  final double sectionHeaderHeight;
  final double listItemHeight;

  ListScaffoldScrollBarConfiguration({
    this.groupScrollBarIndicatorWidth,
    this.groupScrollBarIndicatorWidget,
    this.groupScrollBarWidget,
    this.scrollBarOffset = 0.0,
    this.sectionHeaderHeight = 26.9,
    this.listItemHeight = 59.1,
  });
}

@immutable
class DefaultScaffoldScrollBarConfiguration<T>
    extends ListScaffoldScrollBarConfiguration<T> {
  final List<Widget> sectionItems;
  final List<Widget> sectionIndicatorItems;
  final List<double> sectionPositions;
  final ScrollController scrollController;

  DefaultScaffoldScrollBarConfiguration({
    groupScrollBarIndicatorWidth,
    @required this.sectionItems,
    this.sectionIndicatorItems,
    @required this.sectionPositions,
    this.scrollController,
  }) : super(groupScrollBarIndicatorWidth: groupScrollBarIndicatorWidth);
}
